#!/bin/nawk -f
#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#
#                         A N A L Y Z E     M O D E L
#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# FileName      : analyzeModel.awk
# Author        : Vlad Gershkovich
# Date          : 02/01/2005
# Subject       : Analyzes the output of the 'dialogTest2 ModelDump' tool - 
#                 analyzes the 'SummaryOfCollections__*' files.
# Changes       :
#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

# Copyright(c) QUALLABY 2005                                          PROVISO 4
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

BEGIN {
  FS=","

  PATspace=sprintf(" \t");
  PATspaceLead="^[" PATspace "]*"
  PATspaceTrail= "[" PATspace "]*$"

# mnemonic names for fields
  FLDreq="1"
  FLDse="4"
  FLDpoll="5"

# number of minutes in an hour
  minINhour="60"
}

#---------------------------------------------------------------------------
# trim: strip a string from leading/trailing spaces.
#
function trim(str)
{
   nsub1=sub(PATspaceLead, "", str);
   nsub2=sub(PATspaceTrail, "", str);
   return str;
}

#---------------------------------------------------------------------------
# given epoch time - converter into the human-readable format
#
function convertEpoch( time,    _cmd, _convTime)
{
   if ( time ~ /[^0-9]/ )
      return time;
   
   _cmd= sprintf("/usr/bin/adb <<!\n0t%s=Y\n!\n", time); 
   _cmd | getline _convTime ;
   close(_cmd);
   return _convTime;
}

#---------------------------------------------------------------------------
# report statistics for the just parsed file.
#
function doReport()
{
  printf("######################################\n");
  printf("#### %s\n", trim(convertEpoch(currentFile)));
  printf("######################################\n");

  for( se in seA)
    seC++;

  if ( seC == 0 ) {
     printf("WARNING: there were no SEs and active requests found for [%s]\n", convertEpoch(currentFile)) | "cat 1>&2";
     printf("\n#================================================================\n");
     return;
  }

  printf("Total SEs->[%d]\n\n", seC);

  printf("SEs * metrics = %d\n\n", seTIMESmetrics);
  printf("Avg #metics per SE = %.2f\n\n", seTIMESmetrics / seC);

  printf("%-13s%8s  %10s\n", "#reqs", "frequency", "#measures/hour");
  printf("--------------------------------------\n");

  for( poll in pollA)
    if ( int(poll) != 0 ) {
       pollsINmins= minINhour / poll;
       measuresINhour= pollA[poll] * poll;

       totalMeasures+=measuresINhour;
       
       printf("%-8d -> %3d min -> %10d\n", pollA[poll], pollsINmins, measuresINhour);
    }

    printf("--------------------------------------\n");
    printf("                Total: %10d measures/hour\n", totalMeasures)
    printf("\n#================================================================\n");

    # initialize globals
    seC=totalMeasures=seTIMESmetrics=0
    split("", seA);
    split("", pollA);

}
#---------------------------------------------------------------------------
#############
# MAIN BODY #
#############
# produces reports if we passed the FIRST file supplied to the script - script 
# can take multiple SummaryOfCollections__* files
#
FNR==1 && FNR != NR {
  doReport();
}

# if we are on the FIRST line of a current file - get the DATE portion from the
# file name. 
# The file name should be in the form: SummaryOfCollections__EPOCHTIME.txt 
#                                [e.g. SummaryOfCollections__1107298600.txt]
#
FNR == 1 { 
   currentFile= FILENAME 
   if( match(currentFile,/__[0-9][0-9]*/) ) {
     currentFile=substr(currentFile, RSTART+2, RLENGTH-2);
   }
}

# collect stistics from lines that don't start with '#' [comments] AND that 
# start with the numeric values - the first field is the request ID
#
/^[^#]/ && /^[0-9]/{
  seTIMESmetrics++;
  seA[$FLDse]++;

  pollA[$FLDpoll]++
}

#---------------------------------------------------------------------------

END {
  # take of the LAST file analyzed
  doReport();
}

