#!/bin/nawk -f
#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#
#                         A N A L Y Z E     R E Q U E S T
#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# FileName      : analyzeReq.awk
# Author        : Vlad Gershkovich
# Date          : 02/15/2005
# Subject       : Analyzes the 2 outputs of running 'statGet' -
#                 statGet -l stats > statGet.stats.txt
#                 statGet -l requests > statGet.requests.txt
#
#                 analyzeReq.awk statGet.stats.txt statGet.requests.txt
# Changes       :
#
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

# Copyright(c) QUALLABY 2005                                          PROVISO 4
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
BEGIN {
  # field separator for the "stats" file
  FSstats=":"
  # field separator for the "requests" file
  FSreq=","

  # character separating Element specification in the requests file
  # Elmt=r67501;
  SEPreqElt="="

  # field enumeration for the "stats" file
  FLDstatType="1"
  FLDstatID="2"
  FLDstatName="3"

  # field enumeration for the "requests" file
  FLDreqID="1"

  # patterns for the "stats" file
  PATstatTarget="^Targets$"
  PATstatTasks="^Tasks$"
  PATstatThreadAccess="^Thread Access.*$"
  PATstatThreadAccessInvalid="^[-]$"

  # patterns for the "requests" file
  # get the "Element" specification out of the entire record.
  PATreqElmt="Elmt=[^;][^;]*"
  PATreqElmtComp="[(]SubElmt[)][^_][^_]*"
  PATreqForm="ServiceForm"
  PATreqFormLite="ServiceFormLite"
  PATreqFormComp="ServiceForm:"

  # misc patterns
  PATspace=sprintf(" \t");
  PATspaceLead="^[" PATspace "]*"
  PATspaceTrail= "[" PATspace "]*$"
  PATsubsepARR="(" SUBSEP ")|(^)"

  # set FieldSeparator for the  FIRST file - "stats"
  FS=FSstats
}

#---------------------------------------------------------------------------
# trim: strip a string from leading/trailing spaces.
#
function trim(str)
{
   sub(PATspaceLead, "", str);
   sub(PATspaceTrail, "", str);
   return str;
}

#---------------------------------------------------------------------------
# If we are processing the FIRST file specified on the command line 
# It's always assumed that the "stats" file is specified FIRST
#
 FNR == NR {
   if ( trim($FLDstatType) ~ PATstatTarget ) {
      $FLDstatID=trim($FLDstatID)
      statAtargets[ $FLDstatID ] = ( $FLDstatID in statAtargets ) ? statAtargets[ $FLDstatID ] SUBSEP $0 : $0
   }
   else if ( trim($FLDstatType) ~ PATstatTasks ) {
      $FLDstatID=trim($FLDstatID)
      statAtasks[ $FLDstatID ] = ( $FLDstatID in statAtasks ) ? statAtasks[ $FLDstatID ] SUBSEP $0 : $0
      if ( (trim($FLDstatName) ~ PATstatThreadAccess ) && (trim($NF) !~ PATstatThreadAccessInvalid) && (int(trim($NF)) < 100) ) 
      {
         statAtasksThreadBad[trim($FLDstatID)]=trim($NF);
	 summaryThreads[trim($NF)]++;
      }
   }
       
   next;
}
#---------------------------------------------------------------------------
# If you're starting to process the SECOND file ["requests"] - change the 
# FieldSeparator [FS]
#
FNR==1 && NR!=1 {   FS=FSreq }

#---------------------------------------------------------------------------
# If we are processing the SECOND file specified on the command line 
# It's always assumed that the "requests" file is specified SECOND

$0 ~ PATreqForm {
   idAnum=split(trim($FLDreqID), idA, " ");
   idx=idA[idAnum-1] idA[idAnum];
   reqA[trim(idx)]=$0
   split("", idA)
}

#---------------------------------------------------------------------------
#---------------------------------------------------------------------------
# END of processing - let's produce the report of "BAD" records
#
END {
    for (i in statAtasksThreadBad) {
	printf("#\n# ---------------- Task->[%s] ThreadAccess->[%s] -------------------\n", i, statAtasksThreadBad[i])
        printf("# Task Description:\n")
	tmp=statAtasks[i];
	gsub(PATsubsepARR, "\n#\t", tmp);
	printf("#%s\n", tmp);
	if ( i in reqA ) {
           printf("#\n# Request Description:\n")
	   printf("#\t%s\n", reqA[i])
           if ( reqA[i] ~ PATreqFormLite ) {
	      if (match(reqA[i], PATreqElmt)) {
	         elt=substr(reqA[i],RSTART,RLENGTH)
	         split(elt, eltA, SEPreqElt);
	         elt=trim(eltA[2])
                 #printf("DEBUG: PATreqFormLite: elt->[%s]\n", elt);
              }
           }   
           else if ( reqA[i] ~ PATreqFormComp ) {
                  #printf("DEBUG: PATreqFormComp: req->[%s]\n", reqA[i]);
	          if (match(reqA[i], PATreqElmtComp)) {
	             elt=substr(reqA[i],RSTART,RLENGTH)
	             elt=trim(substr(elt, index(elt, ")")+1))
                     #printf("DEBUG: PATreqFormComp: elt->[%s]\n", elt);
                  }
                }
                else continue;

	   if ( elt in statAtargets ) {
             printf("#\n# Target Description:\n")
             tmp=statAtargets[elt];
	     gsub(PATsubsepARR, "\n#\t", tmp);
	     printf("#%s\n", tmp);
	     summaryPerHost[ elt, statAtasksThreadBad[i]]++;
	   }
	}
     }

    printf("\n##########################################################\n")
    printf("############# SUMMARY #####################################\n")
    printf("\n##########################################################\n")
    printf("Summary ThreadAccess below 100%c - per host\n", "%")
    for (iterHost in summaryPerHost) {
	hostNum=split(iterHost, hostA, SUBSEP);
	host=hostA[1]
	printf("\tHost->[%s]\n", host);
	for (iterTasks in summaryPerHost) {
	    taskNum=split(iterTasks, taskA, SUBSEP);
	    if ( taskA[1] == host ) {
	       printf("\t\t[%-11s]%c availability - [%5d] tasks\n", taskA[2], "%", summaryPerHost[iterTasks]);
	       delete summaryPerHost[iterTasks];
	    }
        }
    }

    printf("\nSummary ThreadAccess below 100%\n")
    for (i in summaryThreads)
	printf("\t[%-11s]%c availability - [%5d] tasks\n", i, "%", summaryThreads[i]);
}
#---------------------------------------------------------------------------
#---------------------------------------------------------------------------
