#!/bin/bash
#set -x

. ./audit.env


#
#	This is the collector number
#
#

allDiscElmts=allDiscElmts.txt
allDiscMetrics=allDiscMetrics.txt

allCollSubElmts=allCollSubElmts.txt
allCollMetrics=allCollMetrics.txt

allFamiliesInSystem=allFamiliesInSystem.txt

if [ ! -d sql.tools ]; then
	mkdir sql.tools;
	fi
	

cat <<_SQL_ >sql.tools/getAllDiscElmts.sql
set pagesize 0
set linesize 100
set feedback off

spool ${allDiscElmts}

Select IPAddress||'|'||Family
	From ( Select Nvl((Select Max(Str_Value)
		From PROP_DESC
		Where Idx_Resource = ( Select Idx_Host
			From SE_DESC
			Where Idx_Ind = s.Idx_Ind )
		And Idx_Metric In ( Select Idx_Ind
			From FRML_DESC
			Where Str_Name = 'ipAddress'
			And Str_Type = 'property' )), '***MissingIPAddress***') IPAddress,
			( Select Ncl_Str_Name
				From Ref_Desc
				Where Ncl_Idx_Ind = s.Ncl_Idx_Ind) Family
			From ( Select Ncl_Idx_Ind, Max(Idx_Ind) Idx_ind
				From SE_DESC
				Where Str_State='on'
				And Idx_Host In ( Select Idx_Ind
					From ELMT_DESC
					Where Int_Collector = ${NEWCOLLECTORNUMBER} )
					Group By Ncl_Idx_Ind ) s )
			Where IPAddress != '***MissingIPAddress***'
	Order By Family, IPAddress
/
Exit 0
_SQL_


cat <<_SQL_ >sql.tools/getAllDiscMetrics.sql
set pagesize 0
set linesize 100
set feedback off

spool ${allDiscMetrics}
select idx_ind||'|'||str_name||'|'
 from frml_desc
where str_type = 'snmp'
  and str_data like '% index%||%'
/
Exit 0
_SQL_

cat <<_SQL_ >sql.tools/getAllCollSubElmts.sql
set pagesize 0
set linesize 100
set feedback off

spool ${allCollSubElmts}
Select  s.Idx_Ind||'|'||
        (Select Str_Value
          From PROP_DESC
         Where Idx_Resource = r.Idx_Host
           And Idx_Metric = ( Select Max(Idx_Ind)
                                From FRML_DESC
                               Where Str_Name = 'ipAddress'
                                 And Str_Type = 'property' ))
       ||'|'||
       ( Select str_instance||'|'||str_alias
           From SE_DESC
          Where Idx_Ind = s.Idx_Ind )
  From ( Select sd.Idx_Ind,
                sd.Idx_Host
           From ELMT_DESC ed,
                SE_DESC sd
          Where ed.Idx_Ind = sd.Idx_Host
            And sd.Str_State='on'
            And ed.Int_Collector = ${NEWCOLLECTORNUMBER}) r,
       ( Select Ncl_Idx_Ind, Max(Idx_Ind) Idx_ind
           From SE_DESC
          Where Str_State='on'
            And Idx_Host In ( Select Idx_Ind
                                From ELMT_DESC
                               Where Int_Collector = ${NEWCOLLECTORNUMBER} )
          Group By Ncl_Idx_Ind ) s
 Where r.Idx_ind = s.Idx_Ind
/
Exit 0
_SQL_

cat <<_SQL_ >sql.tools/getAllCollMetrics.sql
set pagesize 0
set linesize 100
set feedback off

spool ${allCollMetrics}
select distinct(t.idx_formula)||'|'||f.str_name||'|'
     from tsk_desc t, frml_desc f
    where t.idx_formula = f.idx_ind 
      and f.str_type = 'snmp'
/
Exit 0
_SQL_


cat <<_SQL_ >sql.tools/getAllFamiliesInSystem.sql
set pagesize 0
set linesize 100
set feedback off

spool ${allFamiliesInSystem}

select family||'|'||Int_Collector from (
Select Distinct
	( Select Ncl_Str_Name
	From REF_DESC
	Where Ncl_Idx_Ind = sd.Ncl_Idx_Ind ) family,cl.Int_Collector
	From ELMT_DESC ed,
	SE_DESC sd,
		( Select Distinct Substr(Str_Path, 10, Instr(Str_Path, '.', 10)-10) Int_Collector
			From REG\$DATALOAD
			Where Str_Path Like 'DATALOAD.%.PRODUCT_TYPE'
				And Str_Value = 'SNMP' ) cl
			Where ed.Idx_Ind = sd.Idx_Host
			And ed.Int_Collector = cl.Int_Collector
			And sd.Str_State='on'
			And sd.Ncl_Idx_Ind Not In ( Select Ncl_Idx_Ind
				From REF_DESC
				Where Ncl_Str_Name = 'PROVISO_MONITORING' )
Order By family)
/
Exit 0
_SQL_


rm -f allDiscElmts.txt allDiscMetrics.txt allCollSubElmts.txt allCollMetrics.txt allFamiliesInSystem.txt
${ORACLE_HOME}/bin/sqlplus ${DB_USER}/${DB_PASSWD}@${DB_TNSENTRY} @sql.tools/getAllDiscElmts.sql 
${ORACLE_HOME}/bin/sqlplus ${DB_USER}/${DB_PASSWD}@${DB_TNSENTRY} @sql.tools/getAllDiscMetrics.sql
${ORACLE_HOME}/bin/sqlplus ${DB_USER}/${DB_PASSWD}@${DB_TNSENTRY} @sql.tools/getAllCollSubElmts.sql
${ORACLE_HOME}/bin/sqlplus ${DB_USER}/${DB_PASSWD}@${DB_TNSENTRY} @sql.tools/getAllCollMetrics.sql
${ORACLE_HOME}/bin/sqlplus ${DB_USER}/${DB_PASSWD}@${DB_TNSENTRY} @sql.tools/getAllFamiliesInSystem.sql

cat ${allFamiliesInSystem} | nawk -F '|' '{print $1}' | sed -e 's/^ *//' -e 's/ *$//' | sort -u > /tmp/allFamList.txt
cat ${allDiscElmts} | nawk -F '|' '{print $2}' | sed -e 's/^ *//' -e 's/ *$//' | sort -u > /tmp/thisTestFamList.txt

echo "-- Coverage summary ------------------------------------------------------------"
echo "Families covered with that selection:"
cat /tmp/thisTestFamList.txt

echo " "
echo "Families NOT covered with that selection, but available on different collectors:"
echo " (see allFamiliesInSystem.txt for collector numbers)"
diff /tmp/allFamList.txt /tmp/thisTestFamList.txt | grep "< "

rm /tmp/thisTestFamList.txt /tmp/allFamList.txt


 
